// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "PingusResource.hh"
#include "PingusError.hh"
#include "blitter.hh"
#include "globals.hh"

#include "PingusSpotMap.hh"

PingusSpotMap::PingusSpotMap()
{
}

PingusSpotMap::PingusSpotMap(PLF* plf)
{
  ResDescriptor name = plf->get_fg();

  width  = plf->get_width();
  height = plf->get_height();

  switch(name.type) {
  case ResDescriptor::RESOURCE:
    throw PingusError("PingusSpotMap: ResType RESOURCE is not implemented, sorry");
    break;
  case ResDescriptor::FILE:
    load(name.res_name);
    break;
  default:
    throw PingusError("PingusSpotMap: Unknown resource type, bailing out");
    break;
  }
}

PingusSpotMap::~PingusSpotMap(void)
{
  delete provider;
}

// Load the map from a *.psm (Pingu Spot Map) file and load the surfaces
void
PingusSpotMap::load(string filename)
{
  psm_parser.parse(filename);
  psm_parser.load_surfaces();
  surfaces = psm_parser.get_surfaces();

  if (use_lsurfaces) {
    cout << "Creating LockableSurface" << endl;
    lsurface = CL_LockableSurface::create(width, height);
    
    for(vector<CL_Surface*>::size_type i=0; i < surfaces.size(); ++i) 
      put_surface(lsurface, surfaces[i].surface->get_provider(),
		  surfaces[i].x_pos, surfaces[i].y_pos); 
    cout << "Levelmap created" << endl;
  } else {
    provider = new CL_EmptySurfaceProvider(width, height, 1, -1, RGBA8888);
    
    for(vector<surface_data>::size_type i=0; i < surfaces.size(); ++i) 
      put_surface(provider, surfaces[i].surface->get_provider(),
		  surfaces[i].x_pos, surfaces[i].y_pos);
    
    map_surface = CL_Surface::create(provider);
  }
}

// Draws the map with a offset, needed for scrolling
void
PingusSpotMap::draw_offset(int of_x, int of_y, float s)
{
  if (use_lsurfaces) {
    if (s == 1.0) {
      lsurface->put_screen(of_x, of_y);
    } else {
      lsurface->put_screen(of_x, of_y, s);
    }
  } else {
    // Hack
    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_U))
      map_surface->reload();
    // Hack end
    
    if (s == 1.0) {
      map_surface->put_screen(of_x, of_y);
    } else {
      map_surface->put_screen(int(of_x * s), int(of_y * s), s, s);
    }
  }
}

// Returns the width of the map, it is read directly from the *.psm file
int
PingusSpotMap::get_width(void)
{
  return width;
}

// Returns the height of the map, it is read directly from the *.psm file
int 
PingusSpotMap::get_height(void)
{
  return height;
}

void
PingusSpotMap::remove(int x, int y)
{
  if (!use_lsurfaces) {
    unsigned char* buffer;
    if (x > 0 && x < width
	&& y > 0 && y < height) {
      provider->lock();
      buffer = static_cast<unsigned char*>(provider->get_surface_data());
      buffer[y * provider->get_pitch() + (x*4)] = 0;
      provider->unlock();
    } else {
      return;
    }
  }
}

void 
PingusSpotMap::remove(CL_SurfaceProvider* sprovider, int x, int y)
{
  if (!use_lsurfaces) {
    if (debug_actions) {
      put_surface(provider, sprovider, x, y);  
    } else {
      put_alpha_surface(provider, sprovider, x, y);
    }
  } else {
    put_surface(lsurface, sprovider, x, y);  
  }
}

void
PingusSpotMap::put(CL_SurfaceProvider* sprov, int x, int y)
{
  if (use_lsurfaces) {
    put_surface(lsurface, sprov, x, y);
  } else {
    put_surface(provider, sprov, x, y);
  }
}

ColMap*
PingusSpotMap::get_colmap(void)
{
  cout << "PingusSpotMap: Staring ColMap creation" << endl;
  ColMap* col;
  unsigned char* buffer;

  // Allocate the space for the colmap buffer
  // But don't delete it, since the ColMap will do that.
  buffer = new unsigned char[width * height];

  cout << "PingusSpotMap: Clearing buffer" << endl;
  for(int i=0; i < width * height; ++i) {
    buffer[i] = 0;
  }

  // Create a empty ColMap
  col = new ColMap(buffer, width, height);

  for(vector<CL_Surface*>::size_type i=0; i < surfaces.size(); ++i) {
    //cout << "Putting surface";
    col->put(surfaces[i].surface, surfaces[i].x_pos, surfaces[i].y_pos);
  }
  cout << "Returning ColMap" << endl;
  return col;
}

CL_Surface*
PingusSpotMap::get_surface(void)
{
  return map_surface;
}

/* EOF */
